# Esercizio Separazione con iperpiani
# DATI
param nPin;				# Numero dei punti interni
set Pin := 1..nPin;		# Insieme dei punti interni
param nPout;			# Numero dei punti esterni
set Pout := 1..nPout;	# Insieme dei punti esterni
param xin {Pin};		# Ascissa di ogni punto interno
param yin {Pin};		# Ordinata di ogni punto interno
param xout {Pout};		# Ascissa di ogni punto esterno
param yout {Pout};		# Ordinata di ogni punto esterno
param n;				# Max. numero di iperpiani considerati
set R := 1..n;			# Set di iperpiani (rette)
param M := 10;

# VARIABILI
# Coefficienti di ogni retta
var a {R};
var b {R};
var c {R};
var k {R} binary := 1;		# Uso o non uso dell'iperpiano
var w {Pout,R} binary;		# 1: vincolo disattivato

# VINCOLI
# Normalizzazione dei coefficienti se la retta  usata
subject to Normalizzazione {r in R}:
  a[r]^2 + b[r]^2 = k[r];

# I punti interni devono soddisfare tutti i vincoli
subject to Interni {i in Pin, r in R}:
  a[r]*xin[i] + b[r]*yin[i] + c[r] <= 0;

# Ogni punto esterno deve essere fuori da ogni vincolo (disattivabile dalle w)
subject to Esterni {i in Pout, r in R}:
  a[r]*xout[i] + b[r]*yout[i] + c[r] >= 0 - M*w[i,r];

# Numero max vincoli disattivati
subject to Disattivati {i in Pout}:
  sum {r in R} w[i,r] <= sum {r in R} k[r] - 1;

# OBIETTIVO
# Usare il minimo numero di rette
minimize z: sum {r in R} k[r];

#################
data;
#################

param n := 6;

param nPout := 10;
param nPin  := 11;

param: xout   yout :=
1	  5.87	9.16
2	  0.07	6.32
3	  8.11	2.19
4	  9.42	6.71
5	  2.01	7.80
6	  9.41	3.59
7	  7.24	1.71
8	  0.55	5.33
9	  10.0	6.36
10	  2.00  1.50;
	
param: xin  yin :=
 1	  4.18	4.38
 2	  4.72	5.65
 3	  4.92	6.18
 4	  5.61	2.25
 5	  4.90	1.97
 6	  6.83	4.15
 7	  2.12	4.18
 8	  8.13	6.40 
 9	  3.77	5.67
10	  2.46	4.26
11	  2.94	6.37;

var:	a	 b	  c :=
1	   -1	 0    2
2		0   -1    2
3		1    0   -8
4		0    1   -8
5	  -0.7 -0.7   1
6	   0.7  0.7 -10;

end;

#####################################

Minimo numero di rette = 3
Soluzione trovata da Knitro:

        a              b              c
   -0.581941       0.813231      -3.97622 
    0.871924      -0.489641      -4.52044 
   -0.882499      -0.470314       3.44649

